// Parser.java
//
// ICS 142 Winter 2004
// Assignment #2

import java.io.*;



public class Parser
{
	private Scanner scanner;
	private Token nextToken;


	public Parser()
	{
		this.scanner = null;
		nextToken = null;
	}


	public void parse(Scanner scanner)
	{
		this.scanner = scanner;
		readNextToken();

		parseProgram();

		if (!noMoreTokens())
		{
			throwParserError("leftover tokens after program");
		}
	}


	private void parseProgram()
	{

	}


	// nextTokenIs() returns true if the next token is the given token type,
	// false otherwise.
	private boolean nextTokenIs(int tokenType)
	{
		return nextToken != null && nextToken.getTokenType() == tokenType;
	}


	// nextTokenIsNot() returns true if the next token is not the given token
	// type, false otherwise.
	private boolean nextTokenIsNot(int tokenType)
	{
		return nextToken == null || nextToken.getTokenType() != tokenType;
	}


	// noMoreTokens() returns true if there are no more tokens in the input.
	private boolean noMoreTokens()
	{
		return nextToken == null;
	}


	// consumeToken() consumes one token of input, which is expected to be
	// the given token type.  If the next token is not the given token type,
	// a ParserError is thrown, otherwise the next token is read from the
	// input.
	private void consumeToken(int tokenType)
	{
		if (nextTokenIs(tokenType))
		{
			readNextToken();
		}
		else
		{
			throwParserError(Token.getTokenTypeString(tokenType) + " expected");
		}
	}


	// throwParserError() is a handy method that you should use when you want
	// to throw a ParserError.  It automatically throws a ParserError with the
	// given message, as well as the line number and column of the current
	// token.
	private void throwParserError(String message)
	{
		throw new ParserError(
			message,
			nextToken == null ? 1 : nextToken.getLine(),
			nextToken == null ? 1 : nextToken.getColumn());
	}


	// readNextToken() is a utility method used by my consumeToken() method.
	// You shouldn't need to modify this one at all.
	private void readNextToken()
	{
		try
		{
			nextToken = scanner.getNextToken();
		}
		catch (IOException e)
		{
			throwParserError("IOException encountered during parsing: " + e.getMessage());
		}

		// We won't catch a ScannerError here, instead letting it slip
		// out of the parser entirely and back to the Driver.  (Aren't
		// unchecked exceptions glorious? :) )
	}
}
